<?php
/***********************************************************************

  Copyright (C) 2002, 2003, 2004  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


$pun_root = './';
require $pun_root.'include/common.php';
require $pun_root.'include/common_admin.php';


if ($cur_user['status'] < PUN_ADMIN)
	message($lang_common['No permission']);


// Add a "default" forum
if (isset($_POST['add_forum']))
{
	confirm_referer('admin_forums.php');

	$add_to_cat = intval($_POST['add_to_cat']);
	if (empty($add_to_cat))
		message($lang_common['Bad request']);

	$db->query('INSERT INTO '.$db->prefix.'forums (cat_id) VALUES('.$add_to_cat.')') or error('Unable to create forum', __FILE__, __LINE__, $db->error());

	redirect('admin_forums.php', 'Forum added. Redirecting ...');
}


// Delete a forum
else if (isset($_POST['del_forum']) || isset($_POST['del_forum_comply']))
{
	confirm_referer('admin_forums.php');

	$forum_to_delete = intval($_POST['forum_to_delete']);
	if (empty($forum_to_delete))
		message($lang_common['Bad request']);

	if (isset($_POST['del_forum_comply']))	// Delete a forum with all posts
	{
		@set_time_limit(0);

		// Prune all posts and topics (start transaction)
		prune($forum_to_delete, 1, -1);

		// Locate any "orphaned redirect topics" and delete them
		$result = $db->query('SELECT t1.id FROM '.$db->prefix.'topics AS t1 LEFT OUTER JOIN '.$db->prefix.'topics AS t2 ON t1.moved_to=t2.id WHERE t2.id IS NULL AND t1.moved_to IS NOT NULL') or error('Unable to fetch redirect topics', __FILE__, __LINE__, $db->error());
		$num_orphans = $db->num_rows($result);

		if ($num_orphans)
		{
			for ($i = 0; $i < $num_orphans; ++$i)
				$orphans[] = $db->result($result, $i);

			$db->query('DELETE FROM '.$db->prefix.'topics WHERE id IN('.implode(',', $orphans).')') or error('Unable to delete redirect topics', __FILE__, __LINE__, $db->error());
		}

		// Delete the forum (end transaction)
		$db->query('DELETE FROM '.$db->prefix.'forums WHERE id='.$forum_to_delete, PUN_TRANS_END) or error('Unable to delete forum', __FILE__, __LINE__, $db->error());

		redirect('admin_forums.php', 'Forum deleted. Redirecting ...');
	}
	else	// If the user hasn't confirmed the delete
	{
		$result = $db->query('SELECT forum_name FROM '.$db->prefix.'forums WHERE id='.$forum_to_delete) or error('Unable to fetch forum info', __FILE__, __LINE__, $db->error());
		$forum_name = $db->result($result, 0);


		$page_title = pun_htmlspecialchars($pun_config['o_board_title']).' / Admin / Forums';
		require $pun_root.'header.php';

		admin_menu('forums');

?>
<form method="post" action="admin_forums.php">
	<input type="hidden" name="forum_to_delete" value="<?php echo $forum_to_delete ?>">
	<table class="punmain" cellspacing="1" cellpadding="4">
		<tr class="punhead">
			<td class="punhead">Confirm delete forum</td>
		</tr>
		<tr>
			<td class="puncon2">
				<br>&nbsp;Are you sure that you want to delete the forum "<?php echo $forum_name ?>"?<br><br>
				&nbsp;WARNING! Deleting a forum will delete all posts (if any) in that forum!<br><br>
				&nbsp;<input type="submit" name="del_forum_comply" value="Delete">&nbsp;&nbsp;&nbsp;<a href="javascript:history.go(-1)">Go back</a><br><br>
			</td>
		</tr>
	</table>
</form>

<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>
<?php

		require $pun_root.'footer.php';
	}
}


// Update one or more forums
else if (isset($_POST['update']) || isset($_POST['update_all']))
{
	confirm_referer('admin_forums.php');

	$forums_to_process = (isset($_POST['update'])) ? array_keys($_POST['update']) : array_keys($_POST['forum_name']);

	while (list(, $id) = @each($forums_to_process))
	{
		$cur_position = $_POST['position'][$id];
		$cur_forum_name = trim($_POST['forum_name'][$id]);
		$cur_forum_desc = pun_linebreaks(trim($_POST['forum_desc'][$id]));
		$cur_admmod_only = $_POST['admmod_only'][$id];
		$cur_closed = $_POST['closed'][$id];
		$cur_cat_id = intval($_POST['cat_id'][$id]);

		if ($cur_forum_name == '')
			message('You must enter a forum name.');

		if ($cur_position == '' || preg_match('/[^0-9]/', $cur_position))
			message('Position must be a positive integer value.');

		if (empty($cur_cat_id))
			message($lang_common['Bad request']);

		if ($cur_closed != '1') $cur_closed = '0';
		if ($cur_admmod_only != '1') $cur_admmod_only = '0';

		$cur_forum_desc = ($cur_forum_desc != '') ? '\''.escape($cur_forum_desc).'\'' : 'NULL';

		$db->query('UPDATE '.$db->prefix.'forums SET forum_name=\''.escape($cur_forum_name).'\', forum_desc='.$cur_forum_desc.', closed=\''.$cur_closed.'\', admmod_only=\''.$cur_admmod_only.'\', disp_position='.$cur_position.', cat_id='.$cur_cat_id.' WHERE id='.$id) or error('Unable to update forum', __FILE__, __LINE__, $db->error());
	}

	redirect('admin_forums.php', 'Forum(s) updated. Redirecting ...');
}


// Generate an array with all forums and their respective category (used frequently)
$result = $db->query('SELECT c.id AS cid, c.cat_name, f.id AS fid, f.forum_name, f.forum_desc, f.closed, f.admmod_only, f.disp_position FROM '.$db->prefix.'categories AS c LEFT JOIN '.$db->prefix.'forums AS f ON c.id=f.cat_id ORDER BY c.disp_position, c.id, f.disp_position') or error('Unable to fetch category/forum list', __FILE__, __LINE__, $db->error());
$num_forums = $db->num_rows($result);

$forum_list = array();
while ($num_forums--)
	$forum_list[] = $db->fetch_assoc($result);


$page_title = pun_htmlspecialchars($pun_config['o_board_title']).' / Admin / Forums';
require $pun_root.'header.php';

admin_menu('forums');

?>
<form method="post" action="admin_forums.php?action=adddel">
	<table class="punmain" cellspacing="1" cellpadding="4">
		<tr class="punhead">
			<td class="punhead" colspan="2">Add/delete forums</td>
		</tr>
		<tr class="puncon1">
			<td class="puncent" style="width: 50%">
				<br>&nbsp;&nbsp;Add forum to category&nbsp;&nbsp;<select name="add_to_cat" tabindex="1">
<?php

$cur_category = 0;
while (list(, $cur_forum) = @each($forum_list))
{
	if ($cur_forum['cid'] != $cur_category)	// A new category since last iteration?
	{
		echo "\t\t\t\t\t".'<option value="'.pun_htmlspecialchars($cur_forum['cid']).'">'.pun_htmlspecialchars($cur_forum['cat_name']).'</option>'."\n";
		$cur_category = $cur_forum['cid'];
	}
}

?>
				</select>
				&nbsp;&nbsp;<input type="submit" name="add_forum" value=" Add " tabindex="2"><br><br>
			</td>
			<td class="puncent" style="width: 50%">
				<br>&nbsp;&nbsp;Delete forum&nbsp;&nbsp;<select name="forum_to_delete" tabindex="3">
<?php

$cur_category = 0;
@reset($forum_list);
while (list(, $cur_forum) = @each($forum_list))
{
	if ($cur_forum['cid'] != $cur_category)	// A new category since last iteration?
	{
		if (!empty($cur_category))
			echo "\t\t\t\t\t".'</optgroup>'."\n";

		echo "\t\t\t\t\t".'<optgroup label="'.pun_htmlspecialchars($cur_forum['cat_name']).'">'."\n";
		$cur_category = $cur_forum['cid'];
	}

	if ($cur_forum['fid'] != '')
		echo "\t\t\t\t\t\t".'<option value="'.$cur_forum['fid'].'">'.pun_htmlspecialchars($cur_forum['forum_name']).'</option>'."\n";
}

?>
					</optgroup>
				</select>
				&nbsp;&nbsp;<input type="submit" name="del_forum" value="Delete" tabindex="4"><br><br>
			</td>
		</tr>
	</table>
</form>

<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>

<form method="post" action="admin_forums.php?action=edit">
	<table class="punmain" cellspacing="1" cellpadding="4">
		<tr class="punhead">
			<td class="punhead">Edit forums</td>
		</tr>
<?php

$tabindex_count = 5;

$cur_category = 0;
foreach($forum_list as $cur_forum)	// We use foreach instead of each() because we iterate through $forum_list later in the code block
{
	if ($cur_forum['cid'] != $cur_category)	// A new category since last iteration?
	{
		echo "\t\t".'<tr class="puncon3"><td>'.pun_htmlspecialchars($cur_forum['cat_name']).'</td></tr>'."\n";
		$cur_category = $cur_forum['cid'];
	}

	if ($cur_forum['fid'] != '')
	{

?>
		<tr class="puncon1">
			<td>
				<table class="punplain">
					<tr> 
						<td class="punright" style="width: 10%"><b>Position</b></td>
						<td style="width: 32%">&nbsp;<input type="text" name="position[<?php echo $cur_forum['fid'] ?>]" size="3" maxlength="3" value="<?php echo $cur_forum['disp_position'] ?>" tabindex="<?php echo $tabindex_count ?>"></td>
						<td class="punright" style="width: 10%" rowspan="2"><b>Options</b></td>
						<td style="width: 32%; white-space: nowrap">&nbsp;<input type="checkbox" name="admmod_only[<?php echo $cur_forum['fid'] ?>]" value="1" tabindex="<?php echo $tabindex_count+3 ?>"<?php if ($cur_forum['admmod_only'] == '1') echo ' checked'; ?>>&nbsp;Admins/moderators only</td>
						<td class="puncent" style="width: 16%" rowspan="3"><input type="submit" name="update[<?php echo $cur_forum['fid'] ?>]" value="Update" tabindex="<?php echo $tabindex_count+6 ?>"></td>
					</tr>
					<tr>
						<td class="punright"><b>Name</b></td>
						<td>&nbsp;<input type="text" name="forum_name[<?php echo $cur_forum['fid'] ?>]" size="35" maxlength="80" value="<?php echo pun_htmlspecialchars($cur_forum['forum_name']) ?>" tabindex="<?php echo $tabindex_count+1 ?>"></td>
						<td style="white-space: nowrap">&nbsp;<input type="checkbox" name="closed[<?php echo $cur_forum['fid'] ?>]" value="1" tabindex="<?php echo $tabindex_count+4 ?>"<?php if ($cur_forum['closed'] == '1') echo ' checked'; ?>>&nbsp;Closed</td>
					</tr>
					<tr>
						<td class="punright">Description<br>(HTML)</td>
						<td>&nbsp;<textarea name="forum_desc[<?php echo $cur_forum['fid'] ?>]" rows="3" cols="50" tabindex="<?php echo $tabindex_count+2 ?>"><?php echo pun_htmlspecialchars($cur_forum['forum_desc']) ?></textarea></td>
						<td class="punright"><b>Category</b></td>
						<td>
							&nbsp;<select name="cat_id[<?php echo $cur_forum['fid'] ?>]" tabindex="<?php echo $tabindex_count+5 ?>">
<?php

		$cur_category2 = 0;
		@reset($forum_list);
		while (list(, $cur_forum2) = @each($forum_list))
		{
			if ($cur_forum2['cid'] != $cur_category2)	// A new category since last iteration?
			{
				$selected = ($cur_forum['cid'] == $cur_forum2['cid']) ? ' selected' : '';

				echo "\t\t\t\t\t\t\t\t".'<option value="'.$cur_forum2['cid'].'"'.$selected.'>'.pun_htmlspecialchars($cur_forum2['cat_name']).'</option>'."\n";

				$cur_category2 = $cur_forum2['cid'];
			}
		}

?>
							</select>
						</td>
					</tr>
				</table>
			</td>
		</tr>
<?php

	$tabindex_count += 7;

	}
}

?>
		<tr>
			<td class="puncon2cent"><br><input type="submit" name="update_all" value="Update all" tabindex="<?php echo $tabindex_count ?>"><br><br></td>
		</tr>
	</table>
</form>

<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>
<?php

require $pun_root.'footer.php';
