<?php
/***********************************************************************

  Copyright (C) 2002, 2003, 2004  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


$pun_root = './';
require $pun_root.'include/common.php';
require $pun_root.'include/common_admin.php';


if ($cur_user['status'] < PUN_MOD)
	message($lang_common['No permission']);


// Check for upgrade
if ($_GET['action'] == 'check_upgrade')
{
	if (!ini_get('allow_url_fopen'))
		message('Unable to check for upgrade since \'allow_url_fopen\' is disabled on this system.');

	$fp = fopen('http://www.punbb.org/latest_version', 'r');
	$latest_version = trim(@fread($fp, 16));
	@fclose($fp);

	if ($latest_version == '')
		message('Check for upgrade failed for unknown reasons.');

	$cur_version = str_replace('.', '', $pun_config['o_cur_version']);
	$cur_version = (strlen($cur_version) == 2) ? intval($cur_version) * 10 : intval($cur_version);

	$latest_version = str_replace('.', '', $latest_version);
	$latest_version = (strlen($latest_version) == 2) ? intval($latest_version) * 10 : intval($latest_version);

	if ($cur_version >= $latest_version)
		message('You are running the latest version of PunBB.');
	else
		message('A new version of PunBB has been released. You can download the latest version at <a href="http://www.punbb.org/">PunBB.org</a>.');
}


// Show phpinfo() output
else if ($_GET['action'] == 'phpinfo')
{
	phpinfo();
	exit;
}


// Get the server load averages (if possible)
if (@file_exists('/proc/loadavg'))
{
	// We use @ just in case
	$fh = @fopen('/proc/loadavg', 'r');
	$load_averages = @fread($fh, 64);
	@fclose($fh);

	$load_averages = @explode(' ', $load_averages);
	$server_load = $load_averages[0].' '.$load_averages[1].' '.$load_averages[2];
}
else if (preg_match('/averages?: ([0-9\.]+),[\s]+([0-9\.]+),[\s]+([0-9\.]+)/i', @exec('uptime'), $load_averages))
	$server_load = $load_averages[1].' '.$load_averages[2].' '.$load_averages[3];
else
	$server_load = 'Not available';


// Get number of current visitors
$result = $db->query('SELECT COUNT(user_id) FROM '.$db->prefix.'online') or error('Unable to fetch online count', __FILE__, __LINE__, $db->error());
$num_online = $db->result($result, 0);


// Get the database system version
$result = $db->query('SELECT version()') or error('Unable to fetch version info', __FILE__, __LINE__, $db->error());
$db_version = $db->result($result, 0);


// Collect some info about MySQL
if ($db_type == 'mysql')
{
	$db_version = 'MySQL '.$db_version;

	// Calculate total db size/row count (MySQL only)
	$result = $db->query('SHOW TABLE STATUS FROM `'.$db_name.'`') or error('Unable to fetch table status', __FILE__, __LINE__, $db->error());
	$num_tables = $db->num_rows($result);

	while ($num_tables--)
	{
		$status = $db->fetch_row($result);
		$total_records += $status[3];
		$total_size += $status[5] + $status[7];
	}

	$total_size = $total_size / 1024;

	if ($total_size > 1024)
		$total_size = round($total_size / 1024, 2).' MB';
	else
		$total_size = round($total_size, 2).' KB';
}


// See if MMCache or PHPA is loaded
if (function_exists('mmcache'))
	$php_accelerator = '<a href="http://turck-mmcache.sourceforge.net/" target="_blank">Turck MMCache</a>';
else if (isset($_PHPA))
	$php_accelerator = '<a href="http://www.php-accelerator.co.uk/" target="_blank">ionCube PHP Accelerator</a>';
else
	$php_accelerator = 'N/A';
	

$page_title = pun_htmlspecialchars($pun_config['o_board_title']).' / Admin';
require $pun_root.'header.php';

if ($cur_user['status'] == PUN_MOD)
	moderator_menu('index');
else
	admin_menu('index');

?>
<table class="punmain" cellspacing="1" cellpadding="4">
	<tr class="punhead">
		<td class="punhead">Forum administation</td>
	</tr>
	<tr>
		<td class="puncon2">
			Welcome to the PunBB administration control panel. From here you can control vital aspects of the forum. Depending on whether you are an administrator or a moderator you can<br><br>
			&nbsp;- organize categories and forums.<br>
			&nbsp;- set forum-wide options and preferences.<br>
			&nbsp;- control permissions for users and guests.<br>
			&nbsp;- view IP statistics for users.<br>
			&nbsp;- ban users.<br>
			&nbsp;- censor words.<br>
			&nbsp;- set up user ranks.<br>
			&nbsp;- prune old posts.<br>
			&nbsp;- handle post reports.<br><br>
		</td>
	</tr>
</table>

<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>

<table class="punmain" cellspacing="1" cellpadding="4">
	<tr class="punhead">
		<td class="punhead" colspan="2">Statistics</td>
	</tr>
	<tr>
		<td class="puncon1right" style="width: 140px; white-space: nowrap">PunBB version&nbsp;&nbsp;</td>
		<td class="puncon2">
			&nbsp;PunBB <?php echo $pun_config['o_cur_version'] ?> - <a href="admin_index.php?action=check_upgrade">Check for upgrade</a><br><br>
			&nbsp;Developed by Rickard Andersson<br>
			&nbsp;&copy Copyright 2002, 2003, 2004 Rickard Andersson
		</td>
	</tr>
	<tr>
		<td class="puncon1right" style="width: 140px; white-space: nowrap">UNIX load averages&nbsp;&nbsp;</td>
		<td class="puncon2">&nbsp;<?php echo $server_load ?> (<?php echo $num_online ?> users online)</td>
	</tr>
	<tr>
		<td class="puncon1right" style="width: 140px; white-space: nowrap">Environment&nbsp;&nbsp;</td>
		<td class="puncon2">
			&nbsp;Operating system: <?php echo PHP_OS ?><br>
			&nbsp;PHP: <?php echo phpversion() ?> - <a href="admin_index.php?action=phpinfo">Show info</a><br>
			&nbsp;Accelerator: <?php echo $php_accelerator ?>
		</td>
	</tr>
	<tr>
		<td class="puncon1right" style="width: 140px; white-space: nowrap">Database&nbsp;&nbsp;</td>
		<td class="puncon2">
			&nbsp;<?php echo $db_version."\n" ?>
<?php if (isset($total_records) && isset($total_size)): ?>			<br><br>&nbsp;Rows: <?php echo $total_records ?><br>
			&nbsp;Size: <?php echo $total_size."\n" ?>
<?php endif; ?>
		</td>
	</tr>
</table>

<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>
<?php

require $pun_root.'footer.php';